@@ -5,14 +5,18 @@ import re
 import time
 
 import numpy as np
+import pandas as pd
 import pytest
 from skdag import DAG, DAGBuilder
 from skdag.dag.tests.utils import FitParamT, Mult, NoFit, NoTrans, Transf
 from sklearn import datasets
+from sklearn import preprocessing
 from sklearn.base import BaseEstimator, clone
+from sklearn.compose import make_column_selector
 from sklearn.decomposition import PCA
-from sklearn.ensemble import RandomForestClassifier
+from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor
 from sklearn.feature_selection import SelectKBest, f_classif
+from sklearn.impute import SimpleImputer
 from sklearn.linear_model import LinearRegression, LogisticRegression
 from sklearn.pipeline import Pipeline
 from sklearn.preprocessing import StandardScaler
@@ -36,9 +40,8 @@ JUNK_FOOD_DOCS = (
 def test_dag_invalid_parameters():
     # Test the various init parameters of the dag in fit
     # method
-    dag = DAG.from_pipeline([(1, 1)])
-    with pytest.raises(TypeError):
-        dag.fit([[1]], [1])
+    with pytest.raises(KeyError):
+        dag = DAGBuilder().from_pipeline([(1, 1)]).make_dag()
 
     # Check that we can't fit DAGs with objects without fit
     # method
@@ -46,13 +49,13 @@ def test_dag_invalid_parameters():
         "Leaf nodes of a DAG should implement fit or be the string 'passthrough'"
         ".*NoFit.*"
     )
-    dag = DAG.from_pipeline([("clf", NoFit())])
+    dag = DAGBuilder().from_pipeline([("clf", NoFit())]).make_dag()
     with pytest.raises(TypeError, match=msg):
         dag.fit([[1]], [1])
 
     # Smoke test with only an estimator
     clf = NoTrans()
-    dag = DAG.from_pipeline([("svc", clf)])
+    dag = DAGBuilder().from_pipeline([("svc", clf)]).make_dag()
     assert dag.get_params(deep=True) == dict(
         svc__a=None, svc__b=None, svc=clf, **dag.get_params(deep=False)
     )
@@ -67,7 +70,7 @@ def test_dag_invalid_parameters():
     # Test with two objects
     clf = SVC()
     filter1 = SelectKBest(f_classif)
-    dag = DAG.from_pipeline([("anova", filter1), ("svc", clf)])
+    dag = DAGBuilder().from_pipeline([("anova", filter1), ("svc", clf)]).make_dag()
 
     # Check that estimators are not cloned on pipeline construction
     assert dag.named_steps["anova"] is filter1
@@ -76,7 +79,7 @@ def test_dag_invalid_parameters():
     # Check that we can't fit with non-transformers on the way
     # Note that NoTrans implements fit, but not transform
     msg = "All intermediate steps should be transformers.*\\bNoTrans\\b.*"
-    dag2 = DAG.from_pipeline([("t", NoTrans()), ("svc", clf)])
+    dag2 = DAGBuilder().from_pipeline([("t", NoTrans()), ("svc", clf)]).make_dag()
     with pytest.raises(TypeError, match=msg):
         dag2.fit([[1]], [1])
 
@@ -132,7 +135,7 @@ def test_dag_pipeline_init():
     steps = (("transf", Transf()), ("clf", FitParamT()))
     pipe = Pipeline(steps, verbose=False)
     for inp in [pipe, steps]:
-        dag = DAG.from_pipeline(inp)
+        dag = DAGBuilder().from_pipeline(inp).make_dag()
         dag.fit(X, y=None)
         dag.score(X)
 
@@ -148,7 +151,9 @@ def test_dag_methods_anova():
     # Test with Anova + LogisticRegression
     clf = LogisticRegression()
     filter1 = SelectKBest(f_classif, k=2)
-    dag1 = DAG.from_pipeline([("anova", filter1), ("logistic", clf)])
+    dag1 = (
+        DAGBuilder().from_pipeline([("anova", filter1), ("logistic", clf)]).make_dag()
+    )
     dag2 = (
         DAGBuilder()
         .add_step("anova", filter1)
@@ -165,7 +170,11 @@ def test_dag_methods_anova():
 
 def test_dag_fit_params():
     # Test that the pipeline can take fit parameters
-    dag = DAG.from_pipeline([("transf", Transf()), ("clf", FitParamT())])
+    dag = (
+        DAGBuilder()
+        .from_pipeline([("transf", Transf()), ("clf", FitParamT())])
+        .make_dag()
+    )
     dag.fit(X=None, y=None, clf__should_succeed=True)
     # classifier should return True
     assert dag.predict(None)
@@ -182,7 +191,11 @@ def test_dag_fit_params():
 def test_dag_sample_weight_supported():
     # DAG should pass sample_weight
     X = np.array([[1, 2]])
-    dag = DAG.from_pipeline([("transf", Transf()), ("clf", FitParamT())])
+    dag = (
+        DAGBuilder()
+        .from_pipeline([("transf", Transf()), ("clf", FitParamT())])
+        .make_dag()
+    )
     dag.fit(X, y=None)
     assert dag.score(X) == 3
     assert dag.score(X, y=None) == 3
@@ -193,7 +206,7 @@ def test_dag_sample_weight_supported():
 def test_dag_sample_weight_unsupported():
     # When sample_weight is None it shouldn't be passed
     X = np.array([[1, 2]])
-    dag = DAG.from_pipeline([("transf", Transf()), ("clf", Mult())])
+    dag = DAGBuilder().from_pipeline([("transf", Transf()), ("clf", Mult())]).make_dag()
     dag.fit(X, y=None)
     assert dag.score(X) == 3
     assert dag.score(X, sample_weight=None) == 3
@@ -205,7 +218,7 @@ def test_dag_sample_weight_unsupported():
 
 def test_dag_raise_set_params_error():
     # Test dag raises set params error message for nested models.
-    dag = DAG.from_pipeline([("cls", LinearRegression())])
+    dag = DAGBuilder().from_pipeline([("cls", LinearRegression())]).make_dag()
 
     # expected error message
     error_msg = (
@@ -259,6 +272,12 @@ def test_dag_stacking_pca_svm_rf(idx):
     assert dag.predict_log_proba(X).shape == prob_shape
     assert isinstance(dag.score(X, y), (float, np.floating))
 
+    root = dag["log"]
+    for attr in ["n_features_in_", "feature_names_in_"]:
+        if hasattr(root, attr):
+            assert hasattr(dag, attr)
+
+
 
 def test_dag_draw():
     txt = DAGBuilder().make_dag().draw(format="txt")
@@ -304,10 +323,101 @@ def test_dag_draw():
         assert f"{type(est).__name__}" in svg
 
 
+def _dag_from_steplist(steps, **builder_opts):
+    builder = DAGBuilder(**builder_opts)
+    for step in steps:
+        builder.add_step(**step)
+    return builder.make_dag()
+
+
+@pytest.mark.parametrize(
+    "steps",
+    [
+        [
+            {
+                "name": "pca",
+                "est": PCA(n_components=1),
+            },
+            {
+                "name": "svc",
+                "est": SVC(probability=True, random_state=0),
+                "deps": ["pca"],
+            },
+            {
+                "name": "rf",
+                "est": RandomForestClassifier(random_state=0),
+                "deps": ["pca"],
+            },
+            {
+                "name": "log",
+                "est": LogisticRegression(),
+                "deps": ["svc", "rf"],
+            },
+        ],
+    ],
+)
+@pytest.mark.parametrize(
+    "X,y",
+    [datasets.make_blobs(n_samples=200, n_features=10, centers=3, random_state=0)],
+)
+def test_pandas(X, y, steps):
+    dag_np = _dag_from_steplist(steps, infer_dataframe=False)
+    dag_pd = _dag_from_steplist(steps, infer_dataframe=True)
+
+    dag_np.fit(X, y)
+    dag_pd.fit(X, y)
+
+    y_pred_np = dag_np.predict_proba(X)
+    y_pred_pd = dag_pd.predict_proba(X)
+    assert isinstance(y_pred_np, np.ndarray)
+    assert isinstance(y_pred_pd, pd.DataFrame)
+    assert np.allclose(y_pred_np, y_pred_pd)
+
+
+def test_pandas_indexing():
+    X, y = datasets.load_diabetes(return_X_y=True, as_frame=True)
+
+    passcols = ["age", "sex", "bmi", "bp"]
+    preprocessing = (
+        DAGBuilder(infer_dataframe=True)
+        .add_step("imp", SimpleImputer())
+        .add_step("vitals", "passthrough", deps={"imp": passcols})
+        .add_step(
+            "blood",
+            PCA(n_components=2, random_state=0),
+            deps={"imp": make_column_selector("s[0-9]+")},
+        )
+        .add_step("out", "passthrough", deps=["vitals", "blood"])
+        .make_dag()
+    )
+
+    X_tr = preprocessing.fit_transform(X, y)
+    assert isinstance(X_tr, pd.DataFrame)
+    assert (X_tr.index == X.index).all()
+    assert X_tr.columns.tolist() == passcols + ["blood0", "blood1"]
+
+    predictor = (
+        DAGBuilder(infer_dataframe=True)
+        .add_step("rf", RandomForestRegressor(random_state=0))
+        .make_dag()
+    )
+
+    dag = preprocessing.join(
+        predictor,
+        edges=[("out", "rf")],
+    )
+
+    y_pred = dag.fit_predict(X, y)
+
+    assert isinstance(y_pred, pd.Series)
+    assert (y_pred.index == y.index).all()
+    assert y_pred.name == dag.leaves_[0].name
+
+
 @parametrize_with_checks(
     [
-        DAG.from_pipeline([("ss", StandardScaler())]),
-        DAG.from_pipeline([("lr", LinearRegression())]),
+        DAGBuilder().from_pipeline([("ss", StandardScaler())]).make_dag(),
+        DAGBuilder().from_pipeline([("lr", LinearRegression())]).make_dag(),
         (
             DAGBuilder()
             .add_step("pca", PCA(n_components=1))
